import React, { useState, useEffect } from "react";

import TableConfig from "./DynamicTable/interfaces/TableConfig";
import TableActions from "./DynamicTable/interfaces/TableActions";
import TableTexts from "./DynamicTable/interfaces/TableTexts";

import PaginationBar from "./DynamicTable/components/PaginationBar";

import Filters from "./DynamicTable/components/Filters";
import { FilterList, HighlightOff, Clear } from "@material-ui/icons";
import {
    Button,
    Table,
    TableBody,
    TableHead,
    TableRow,
    TableCell,
    TableSortLabel,
    withStyles,
    Typography,
    TextField,
    InputAdornment,
    IconButton
} from "@material-ui/core";

import styles from "./DynamicTable/styles";

export interface DynamicTableProps {
    isLoading: boolean;
    tableData: any[];
    tableConfig: TableConfig[];
    tableActions: TableActions[];
    currentPage: number;
    itemsPerPage: number;
    language: string;
    onPageChange: any;
    classes: any;
    tableTexts: TableTexts;
    loader: any;
}

const DynamicTable: React.SFC<DynamicTableProps> = ({
    isLoading,
    tableData,
    tableConfig,
    tableActions,
    currentPage,
    itemsPerPage,
    language,
    onPageChange,
    tableTexts,
    classes,
    loader
}) => {
    const [filterIsOpenState, setFilterIsOpenState] = useState({});
    const [filteredData, setFilteredData] = useState([]);
    const [filterState, setFilterState] = useState({});
    const [sortState, setSortState] = useState<{
        orderBy: any;
        sortingOrder: any;
    }>({ orderBy: null, sortingOrder: null });
    const [groupsState, setGroupsState] = useState({});
    const [mainFilter, setMainFilter] = useState("");
    const [isClicked, setClicked] = useState(false);
    // const [isInit, setInit] = useState(true);

    let tableRef;

    useEffect(() => {
        // if (isInit) {
        //     setInit(false);
        //     return;
        // }
        if (
            sessionStorage.getItem("isRedirected") &&
            sessionStorage.getItem("isReturnFromRedirect")
        ) {
            sessionStorage.removeItem("isRedirected");
            sessionStorage.removeItem("isReturnFromRedirect");
            return;
        }
        sessionStorage.removeItem("isRedirected");
        sessionStorage.removeItem("isReturnFromRedirect");
        const tempGroupState = {};
        let initialSort: any = { orderBy: null, sortingOrder: null };
        tableData.forEach(item => {
            tableConfig.forEach(config => {
                if (config.sort) {
                    initialSort = {
                        orderBy: config.value,
                        sortingOrder: config.sort
                    };
                }
                if (
                    tempGroupState[config.value] &&
                    tempGroupState[config.value][item[config.value]]
                ) {
                    return;
                }
                tempGroupState[config.value] = {
                    ...tempGroupState[config.value],
                    [item[config.value]]: false
                };
            });
        });
        setGroupsState(tempGroupState);
        setFilterIsOpenState({});
        setMainFilter("");
        setSortState(initialSort);
        // eslint-disable-next-line
    }, [tableData]);

    useEffect(() => {
        setFilteredData(
            filterByMainSearch(filterData(sortData(tableData))).slice(
                currentPage * itemsPerPage,
                currentPage * itemsPerPage + itemsPerPage
            )
        );
        // if (filteredData && filteredData.length === 0 && !isInit) {
        if (filteredData && filteredData.length === 0) {
            onPageChange(0);
        }
        // eslint-disable-next-line
    }, [currentPage, groupsState, sortState, mainFilter]);

    function sortData(data) {
        let tempData: any = data;
        tempData = tempData.sort((a: any, b: any) => {
            a = a[sortState.orderBy];
            b = b[sortState.orderBy];
            if (!isNaN(+a) && !isNaN(+b)) {
                a = +a;
                b = +b;
            }
            if (a === null) {
                return 1;
            }
            if (b === null) {
                return -1;
            }
            let res = a < b ? -1 : a === b ? 0 : 1;
            res = sortState.sortingOrder === "desc" ? res * -1 : res;
            return res;
        });
        return tempData;
    }

    function filterData(data) {
        const tempData = data;
        if (!checkIfAnyFilterIsOn()) {
            return tempData;
        }
        return tempData.filter(row => {
            let allGood = false;
            Object.keys(groupsState).forEach(key => {
                if (groupsState[key][row[key]]) {
                    allGood = true;
                }
            });
            return allGood;
        });
    }

    function filterByMainSearch(data) {
        let tempData = data;
        tempData = tempData.filter((row, index) => {
            let allGood = false;
            tableConfig.forEach(item => {
                if (
                    formatText(tempData[index][item.value]).indexOf(
                        formatText(mainFilter)
                    ) > -1
                ) {
                    allGood = true;
                }
            });
            return allGood;
        });
        return tempData;
    }

    function filterGroupData(data, field) {
        const tempData = data;
        return tempData.filter(item => {
            if (
                formatText(item).indexOf(filterState[field]) === -1 &&
                filterState[field]
            ) {
                return false;
            }
            return true;
        });
    }

    function formatText(text) {
        if (typeof text === "string") {
            return text.toLowerCase();
        }
        return String(text);
    }

    function handleSortChange(title) {
        if (sortState.orderBy !== title) {
            setSortState({ orderBy: title, sortingOrder: "desc" });
        } else {
            switch (sortState.sortingOrder) {
                case "desc":
                    setSortState({ orderBy: title, sortingOrder: "asc" });
                    break;
                case "asc":
                    setSortState({ orderBy: title, sortingOrder: "desc" });
                    break;
                default:
                    break;
            }
        }
    }

    function handleFilterChange(field, input) {
        if (typeof input === "string") {
            input = input.toLocaleLowerCase();
        }
        setFilterState({
            ...filterState,
            [field]: input
        });
    }

    function checkIfAnyFilterIsOn() {
        let state = false;
        Object.keys(groupsState).forEach(group => {
            Object.keys(groupsState[group]).forEach(key => {
                if (groupsState[group][key]) {
                    state = true;
                }
            });
        });
        return state;
    }

    function handleFilterIsOpenState(field) {
        if (filterIsOpenState[field]) {
            handleFilterChange(field, "");
        }
        setFilterIsOpenState({
            [field]: !filterIsOpenState[field]
        });
        handleFilterChange(field, "");
    }

    function handleGroupsChanged(field, value) {
        setGroupsState({
            ...groupsState,
            [field]: {
                ...groupsState[field],
                [value]: !groupsState[field][value]
            }
        });
    }

    function handleGroupsChangedAll(key, value) {
        const tempState = { ...groupsState };
        Object.keys(tempState[key]).forEach(val => {
            tempState[key][val] = value;
        });
        setGroupsState(tempState);
    }

    function resetAllFilters() {
        const tempState = { ...groupsState };
        Object.keys(tempState).forEach(key => {
            Object.keys(tempState[key]).forEach(val => {
                tempState[key][val] = false;
            });
        });
        setGroupsState(tempState);
        setFilterIsOpenState({});
    }

    if (isLoading) {
        return loader;
    }

    return (
        <div className={classes.content_container}>
            <div className={classes.top_section}>
                <div>
                    <TextField
                        className={classes.header_input}
                        onChange={event => setMainFilter(event.target.value)}
                        variant="standard"
                        placeholder={tableTexts.mainSearchPlaceholder}
                        value={mainFilter}
                        InputProps={{
                            endAdornment: (
                                <InputAdornment position="end">
                                    {mainFilter.trim().length > 0 ? (
                                        <IconButton
                                            className={
                                                classes.buttonPlaceholder
                                            }
                                            color="inherit"
                                            aria-label="Toggle password visibility"
                                            onClick={() => setMainFilter("")}
                                        >
                                            <Clear fontSize="small" />
                                        </IconButton>
                                    ) : (
                                        <span
                                            className={
                                                classes.buttonPlaceholder
                                            }
                                        />
                                    )}
                                </InputAdornment>
                            )
                        }}
                    />
                </div>
                {checkIfAnyFilterIsOn() && (
                    <Button
                        variant="outlined"
                        onClick={() => resetAllFilters()}
                        color="default"
                        size="small"
                    >
                        <span>{tableTexts.mainSeachCleanText}</span>{" "}
                        <HighlightOff fontSize="default" />
                    </Button>
                )}
            </div>
            <div
                className={classes.table_container}
                onMouseDown={() => setClicked(true)}
                onMouseUp={() => setClicked(false)}
                onMouseMove={e => {
                    if (isClicked) {
                        e.preventDefault();
                        tableRef.scrollLeft += e.movementX * -1;
                    }
                }}
                ref={ref => (tableRef = ref)}
            >
                <Table>
                    <TableHead>
                        <TableRow>
                            {tableConfig.map((item, index) => (
                                <TableCell
                                    className={classes.tableHeaderText}
                                    key={`tableHeader-${index}`}
                                >
                                    <div className={classes.header_item}>
                                        <div className={classes.header_row}>
                                            <div
                                                className={classes.filter_icon}
                                            >
                                                <FilterList
                                                    className={
                                                        classes.filter_icon
                                                    }
                                                    onClick={() =>
                                                        handleFilterIsOpenState(
                                                            item.value
                                                        )
                                                    }
                                                />
                                            </div>
                                            <TableSortLabel
                                                hideSortIcon={
                                                    sortState.orderBy !==
                                                    item.value
                                                }
                                                active={
                                                    sortState.orderBy ===
                                                    item.value
                                                }
                                                direction={
                                                    sortState.sortingOrder ||
                                                    "desc"
                                                }
                                                onClick={() =>
                                                    handleSortChange(item.value)
                                                }
                                            >
                                                <div
                                                    className={
                                                        classes.text_container
                                                    }
                                                >
                                                    <div
                                                        className={
                                                            classes.header_text
                                                        }
                                                    >
                                                        {item.name}
                                                    </div>
                                                </div>
                                            </TableSortLabel>
                                        </div>
                                        {filterIsOpenState[item.value] && (
                                            <Filters
                                                classes={classes}
                                                handleFilterChange={
                                                    handleFilterChange
                                                }
                                                item={item}
                                                filterGroupData={
                                                    filterGroupData
                                                }
                                                handleGroupsChangedAll={
                                                    handleGroupsChangedAll
                                                }
                                                handleGroupsChanged={
                                                    handleGroupsChanged
                                                }
                                                index={index}
                                                groupsState={groupsState}
                                                handleFilterIsOpenState={
                                                    handleFilterIsOpenState
                                                }
                                                isLast={
                                                    tableConfig.length ===
                                                    index + 1
                                                }
                                                tableTexts={tableTexts}
                                            />
                                        )}
                                    </div>
                                </TableCell>
                            ))}
                            {tableActions &&
                                tableActions.map((action, actionIndex) => (
                                    <TableCell
                                        className={classes.tableHeaderText}
                                        key={`tableHeaderAction-${actionIndex}`}
                                    >
                                        <Typography
                                            variant="h6"
                                            className={classes.header_text}
                                        >
                                            {action.name}
                                        </Typography>
                                    </TableCell>
                                ))}
                        </TableRow>
                    </TableHead>
                    <TableBody>
                        {!isLoading && filteredData.length === 0 ? (
                            <TableRow>
                                <TableCell
                                    colSpan={
                                        tableActions
                                            ? tableConfig.length +
                                              tableActions.length
                                            : tableConfig.length
                                    }
                                >
                                    {tableTexts.tableEmptyMessage}
                                </TableCell>
                            </TableRow>
                        ) : (
                            filteredData.map((row, rowIndex) => (
                                <TableRow key={`tableRow-${rowIndex}`}>
                                    {tableConfig.map((config, colIndex) => (
                                        <TableCell
                                            className={classes.body_text}
                                            key={`tableCell-${rowIndex}-${colIndex}`}
                                        >
                                            {typeof config.calc === "function"
                                                ? config.calc(
                                                      row[config.value],
                                                      row
                                                  )
                                                : row[config.value]}
                                        </TableCell>
                                    ))}
                                    {tableActions &&
                                        tableActions.map(
                                            (action, actionIndex) => (
                                                <TableCell
                                                    className={
                                                        classes.body_text
                                                    }
                                                    key={`tableCellAction-${rowIndex}-${actionIndex}`}
                                                >
                                                    {action.render(row)}
                                                </TableCell>
                                            )
                                        )}
                                </TableRow>
                            ))
                        )}
                    </TableBody>
                </Table>
            </div>
            <div className={classes.table_footer}>
                <PaginationBar
                    pages={Math.ceil(
                        filterByMainSearch(filterData(tableData)).length /
                            itemsPerPage
                    )}
                    currentPage={currentPage}
                    onPageChange={({ selected }) => onPageChange(selected)}
                    language={language}
                />
            </div>
        </div>
    );
};

export default withStyles(styles)(DynamicTable);
